// TerrainFile.cpp: implementation of the CTerrainFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "vscompiler.h"
#include "TerrainFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTerrainFile::CTerrainFile(char *fileName,int dimX,int dimZ)
{
	int i,j;

	// save dimensions

	m_dimX=dimX;
	m_dimZ=dimZ;

	// clear the data buffer pointer

	m_heightBuf=NULL;

	// open the file

	FILE *file=fopen(fileName,"rb");

	// were we successful?

	if (file) {

		// yes, allocate data bufffer

		m_heightBuf=new float[dimX*dimZ];

		// allocate row buffer

		BYTE *buf=new BYTE[dimX];

		// pre-calculate float values for byte values

		float byteVal[256];
		for (i=0;i<256;i++)
			byteVal[i]=(float) i/(float) 255;

		// init extrema variables outside of possible range

		float minVal=2.0f;
		float maxVal=-2.0f;

		// set up buffer pointer to use for iterative storage

		float *bufPtr=m_heightBuf;

		// loop through the rows

		for(i=0;i<m_dimZ;i++) {

			// read in a row

			fread(buf,m_dimX,1,file);

			// loop through the columns

			for (j=0;j<m_dimX;j++) {

				// convert byte value to float

				float val=byteVal[buf[j]];

				// store the value

				*bufPtr=val;

				// check against extrema and adjust extrema if necessary

				if (val<minVal)
					minVal=val;
				else if (val>maxVal)
					maxVal=val;

				// increment height buffer pointer

				bufPtr++;
			}
		}

		// de-allocate row buffer

		delete buf;

		// calculate scaling factor to normalize to range of 0.0 to 1.0

		float scale=1.0f/(maxVal-minVal);

		// loop through the entire array and normalize

		bufPtr=m_heightBuf;
		for (i=0;i<m_dimX*m_dimZ;i++) {
			*bufPtr-=minVal;
			*bufPtr*=scale;
			bufPtr++;
		}

		// close the file

		fclose(file);
	}
}

CTerrainFile::~CTerrainFile()
{
	// de-allocate height buffer

	SAFE_DELETE(m_heightBuf);
}

void CTerrainFile::GetHeightsToArray(float *pfDest, 
									 int srcX,int srcZ,
									 int srcDimX,int srcDimZ, 
									 int destDimX, int destDimZ,
									 int destStride)
{
	int i,j;

	// abort if no data

	if (!m_heightBuf)
		return;

	// get pointer to start of destination buffer

	float *dest=pfDest;

	// get pointer to first referenced data

	float *srcRow=&m_heightBuf[srcZ*m_dimX+srcX];

	// are the destination and source rectangles have different different dimensions?

	if (srcDimX!=destDimX||srcDimZ!=destDimZ)
		goto stretch;

	// loop through the distination rows

	for (i=0;i<destDimZ;i++) {

		// copy data to the destination

		memcpy(dest,srcRow,sizeof(float)*destDimX);

		// increment destination pointer to next row

		dest+=destStride;

		// increment source row pointer

		srcRow+=m_dimX;
	}

	// return to caller

	return;

stretch:

	// loop through the destination rows

	for (i=0;i<destDimZ;i++) {

		// calculate location of row within array as float

		float z=srcZ+srcDimZ*((float) i/(float) (destDimZ-1));

		// extract integer index and floating point weight

		int iz1,iz2;
		float fz1,fz2;
		iz1=(int) z;
		if (iz1>=srcZ+srcDimZ) {
			iz2=iz1=srcZ+srcDimZ;
			fz1=1.0f;
			fz2=0.0f;
		} else {
			iz2=iz1+1;
			fz2=z-floorf(z);
			fz1=1.0f-fz2;
		}
		for (j=0;j<destDimX;j++) {

			// calculate location of column within array as float

			float x=srcX+srcDimX*((float) j/(float) (destDimX-1));

			// extract integer index and floating point weight

			int ix1,ix2;
			float fx1,fx2;
			ix1=(int) x;
			if (ix1>=srcX+srcDimX) {
				ix2=ix1=srcX+srcDimX;
				fx1=1.0f;
				fx2=0.0f;
			} else {
				ix2=ix1+1;
				fx2=x-floorf(x);
				fx1=1.0f-fx2;
			}

			// find interpolated value and store in destination

			float val1,val2;
			val1=fx1*m_heightBuf[iz1*m_dimX+ix1]+fx2*m_heightBuf[iz1*m_dimX+ix2];
			val2=fx1*m_heightBuf[iz2*m_dimX+ix1]+fx2*m_heightBuf[iz2*m_dimX+ix2];
			*dest=val1*fz1+val2*fz2;

			// increment destination pointer

			dest++;
		}

		// offset to start of next destination row

		dest+=destStride-destDimX;
	}
}
