//-----------------------------------------------------------------------------
// File: VSCompiler.h
//
// Desc: Header file VSCompiler sample app
//-----------------------------------------------------------------------------
#ifndef AFX_VSCOMPILER_H__0A9A27A7_673D_414C_B91E_A7395CC8B966__INCLUDED_
#define AFX_VSCOMPILER_H__0A9A27A7_673D_414C_B91E_A7395CC8B966__INCLUDED_

#include "TerrainFile.h"	// Added by ClassView
#if _MSC_VER >= 1000
#pragma once
#endif
#ifndef __AFXWIN_H__
#error include 'stdafx.h' before including this file
#endif



//-----------------------------------------------------------------------------
// Defines, and constants
//-----------------------------------------------------------------------------

#define DXAPP_KEY        TEXT("Software\\XZone\\LandShader")

#define MAX_SPLINES		64
#define LAND_XZ_SCALE	8192.0f
#define LAND_Y_SCALE	2048.0f

// Custom D3D vertex format used by for spline patch generation
// contains X and Z coordinates, in the form of:
//  1,x,x^2,x^3  and 1,z,z^2,z^3

typedef struct _SPLINEVERTEX
{
	D3DXVECTOR4 x;
	D3DXVECTOR4 z;

	_SPLINEVERTEX(float fX,float fZ) {
		x.x=1.0f; x.y=fX; x.z=fX*fX; x.w=fX*fX*fX;
		z.x=1.0f; z.y=fZ; z.z=fZ*fZ; z.w=fZ*fZ*fZ;
	}
	_SPLINEVERTEX() {}

} SPLINEVERTEX,*LPSPLINEVERTEX;


//-----------------------------------------------------------------------------
// Name: class CAppForm
// Desc: CFormView-based class which allows the UI to be created with a form
//       (dialog) resource. This class manages all the controls on the form.
//-----------------------------------------------------------------------------
class CAppForm : public CFormView, public CD3DApplication
{
public:
    BOOL                    m_bLoadingApp;          // TRUE, if the app is loading
    LPDIRECT3DVERTEXBUFFER8 m_pVB;                  // Vextex buffer 
	LPDIRECT3DINDEXBUFFER8 m_pIB;
	DWORD m_hVertexShader;

    FLOAT                   m_fRotY;
	FLOAT					m_fRotVelY;
	FLOAT					m_fFwdVel;

	D3DXVECTOR3				m_vDir;
	D3DXVECTOR3				m_vPos;

	D3DXMATRIX m_matView;
	D3DXMATRIX m_matWorld;

private:
    HWND    m_hwndRenderWindow;
    HWND    m_hwndRenderFullScreen;
    HWND    m_hWndTopLevelParent;

    HRESULT ConfirmDevice( D3DCAPS8*,DWORD,D3DFORMAT );
    HRESULT OneTimeSceneInit();
    HRESULT InitDeviceObjects();
    HRESULT RestoreDeviceObjects();
    HRESULT FrameMove();
    HRESULT InvalidateDeviceObjects();
    HRESULT DeleteDeviceObjects();
    HRESULT FinalCleanup();
    virtual HRESULT AdjustWindowForChange();

    HRESULT RenderText();

    VOID    ReadSettings();
    VOID    WriteSettings();

    VOID    UpdateUIForDeviceCapabilites();

protected:
    DECLARE_DYNCREATE(CAppForm)

             CAppForm();
    virtual  ~CAppForm();

public:
    BOOL IsReady() { return m_bReady; }
    VOID RenderScene() { Render3DEnvironment(); }
    HRESULT CheckForLostFullscreen();

    //{{AFX_DATA(CAppForm)
	enum { IDD = IDD_FORMVIEW };
	CString	m_strFps;
	CString	m_strPatchsize;
	CString	m_strSplinesaxis;
	CString	m_strTriangles;
	CString	m_strVertices;
	CString m_strDeviceInfo;
	//}}AFX_DATA

    //{{AFX_VIRTUAL(CAppForm)
	public:
    virtual void OnInitialUpdate();
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);
    virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
	//}}AFX_VIRTUAL

public:
    HRESULT Render();
	float m_fTextUpdateTime;
	BOOL m_bAutoRot;
	void InitViewOrientation();
	D3DXVECTOR4 m_vQuadScaling;
	D3DXVECTOR4 m_vQuadBase;
	float m_yScaling[MAX_SPLINES];
	float m_yOffset[MAX_SPLINES];
	float m_fHeight[MAX_SPLINES][MAX_SPLINES];
	float m_fLight[MAX_SPLINES][MAX_SPLINES];
	float m_fQuadSize;
	void PrecalcCatmullRom(D3DXVECTOR4 *v,float p0,float p1,float p2,float p3);
	float CalcHeight(float x,float z);
	float CalcQuad(float x,float z,D3DXMATRIX *quad);
	void CalcQuadScaling();
	int m_numSplines;
	int m_patchWidth;
	CTerrainFile *m_terrainFile;
	void CalcQuadMatrices(D3DXMATRIX *pMat,float *pData,int numRows,int numCols);
	HRESULT CreateBuffers();
	void CreateGeometry(BOOL bNewLand=TRUE);
	HRESULT CreateShader();
	HRESULT RenderTiles(int numTiles,int numSamples=1);
    //{{AFX_MSG(CAppForm)
    afx_msg void OnToggleFullScreen();
    afx_msg void OnChangeDevice();
	afx_msg void OnDone();
	afx_msg void OnWire();
	afx_msg void OnNewLandscape();
	afx_msg void OnLargerpatches();
	afx_msg void OnSmallerpatches();
	afx_msg void OnMoresplines();
	afx_msg void OnLesssplines();
	afx_msg void OnTogglerot();
	afx_msg void OnChangeview();
	afx_msg void OnCapturescreen();
	//}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};




//-----------------------------------------------------------------------------
// Name: class CAppDoc
// Desc: Overridden CDocument class needed for the CFormView
//-----------------------------------------------------------------------------
class CAppDoc : public CDocument
{
protected:
    DECLARE_DYNCREATE(CAppDoc)

// Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CAppDoc)
    public:
    //}}AFX_VIRTUAL

// Implementation
    //{{AFX_MSG(CAppDoc)
        // NOTE - the ClassWizard will add and remove member functions here.
        //    DO NOT EDIT what you see in these blocks of generated code !
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};




//-----------------------------------------------------------------------------
// Name: class CAppFrameWnd
// Desc: CFrameWnd-based class needed to override the CFormView's window style
//-----------------------------------------------------------------------------
class CAppFrameWnd : public CFrameWnd
{
protected:
    DECLARE_DYNCREATE(CAppFrameWnd)
public:
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CAppFrameWnd)
    public:
    virtual BOOL PreCreateWindow( CREATESTRUCT& cs );
    virtual BOOL PreTranslateMessage(MSG* pMsg);
    virtual BOOL LoadFrame(UINT nIDResource, DWORD dwDefaultStyle = WS_OVERLAPPEDWINDOW | FWS_ADDTOTITLE, CWnd* pParentWnd = NULL, CCreateContext* pContext = NULL);
    //}}AFX_VIRTUAL

protected:
    //{{AFX_MSG(CAppFrameWnd)
    afx_msg void OnChangeDevice();
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};




//-----------------------------------------------------------------------------
// Name: class CApp
// Desc: Main MFC application class derived from CWinApp.
//-----------------------------------------------------------------------------
class CApp : public CWinApp
{
public:

// Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CApp)
    public:
    virtual BOOL InitInstance();
    virtual BOOL OnIdle( LONG );
    //}}AFX_VIRTUAL

// Implementation
    //{{AFX_MSG(CApp)
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};


#endif



