//-----------------------------------------------------------------------------
// File: VSCompiler.cpp
//
// Desc: DirectX MFC dialog application created by the DirectX AppWizard
//-----------------------------------------------------------------------------
#define STRICT
#include "stdafx.h"
#include <windows.h>
#include <afxole.h>
#include <basetsd.h>
#include <math.h>
#include <stdio.h>
#include "D3DFile.h"
#include "D3DUtil.h"
#include "DXUtil.h"
#include "VSCompiler.h"
#include "terrainFile.h"
#include "mmsystem.h"

// --- shader input ---//

//c0-c3       world/view/proj matrix
//c4            quad size
//c5            quad origin
//c6-c9       position spline matrix
//c10-c13   lighting spline matrix

const char g_szShader[]=
"vs.1.0                                                                     \n"\
"def c14,0.7,0.4,0.3,1.0             ; start color                          \n"\
"def c15,-0.6,0.3,-0.2,0.0           ; end color                            \n"\
"mov r0.xw,v0.yx                      ; get coordinates x,x,z,1 -> r0        \n"\
"mov r0.z,v1.y                                                              \n"\
"m4x4 r3,v0,c[6]                     ; calc vector -> r3                    \n"\
"dp4 r0.y,r3,v1                      ; get height                           \n"\
"sub r7.y,c[14].w,r0.y                  ; calc color                           \n"\
"mul r7.y,r7.y,r7.y                                                         \n"\
"mul r4,c[15],r7.y                                                          \n"\
"add r4,r4,c[14]                                                            \n"\
"m4x4 r7,v0,c[10]                    ;calc lighting                         \n"\
"dp4 r6.y,r7,v1                                                             \n"\
"mul oD0.xyz,r4.xyz,r6.y             ; calculate final diffuse              \n"\
//"mul r0,r0,c[4]                      ; scale vertex                         \n"\//
//"add r0,r0,c[5]                      ; translate vertex                     \n"\//
"m4x4 oPos,r0,c[0]                   ; transform vertex to screen           \n";

//-----------------------------------------------------------------------------
// Name: CAppForm::InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
HRESULT CAppForm::InitDeviceObjects()
{
	return S_OK;
}

HRESULT CAppForm::CreateBuffers()
{
	if (m_pVB)
		m_pVB->Release();
	m_pVB=NULL;
	if (m_pIB)
		m_pIB->Release();
	m_pIB=NULL;

    HRESULT hr;

    // Create the vertex buffer
    if( FAILED( hr = m_pd3dDevice->CreateVertexBuffer( m_patchWidth*m_patchWidth*sizeof(SPLINEVERTEX),
                                                  D3DUSAGE_WRITEONLY, 0,
                                                  D3DPOOL_DEFAULT, &m_pVB ) ) )
        return DXTRACE_ERR_NOMSGBOX( "CreateVertexBuffer", hr );

    SPLINEVERTEX* pVertices;

    if( FAILED( hr = m_pVB->Lock( 0, 0, (BYTE**)&pVertices, 0 ) ) )
        return DXTRACE_ERR_NOMSGBOX( "Lock", hr );

	int i,j;
	for (i=0;i<m_patchWidth;i++) {
		float x=i/(float) (m_patchWidth-1);
		for (j=0;j<m_patchWidth;j++) {
			float z=j/(float) (m_patchWidth-1);
			*pVertices=SPLINEVERTEX(x,z);
			pVertices++;
		}
	}

    m_pVB->Unlock();

	// Create the index buffer
    if( FAILED( hr = m_pd3dDevice->CreateIndexBuffer( (m_patchWidth-1)*(m_patchWidth-1)*2*3*sizeof(WORD),
                                                  D3DUSAGE_WRITEONLY, D3DFMT_INDEX16 ,
                                                  D3DPOOL_DEFAULT, &m_pIB ) ) )
        return DXTRACE_ERR_NOMSGBOX( "CreateIndexBuffer", hr );

	WORD* pInd;
	if( FAILED( hr = m_pIB->Lock( 0, 0, (BYTE**)&pInd, 0 ) ) )
		return DXTRACE_ERR_NOMSGBOX( "Lock Ind", hr );

	for (i=0;i<m_patchWidth-1;i++) {
		for (j=0;j<m_patchWidth-1;j++) {
			int base=i*m_patchWidth+j;
			*pInd=base;								pInd++;
			*pInd=base+1;							pInd++;
			*pInd=base+m_patchWidth+((i+j)&0x01);	pInd++;
			*pInd=base+m_patchWidth;				pInd++;
			*pInd=base+1-((i+j)&0x01);				pInd++;
			*pInd=base+m_patchWidth+1;				pInd++;
		}
	}

    m_pIB->Unlock();

    return S_OK;
}

D3DXMATRIX splineMat[MAX_SPLINES-3][MAX_SPLINES-3];
D3DXMATRIX splineNormMat[MAX_SPLINES-3][MAX_SPLINES-3];

void CAppForm::PrecalcCatmullRom(D3DXVECTOR4 *v,float p0,float p1,float p2,float p3)
{
	v->x=p1;
	v->y=(p2-p0)*0.5f;
	v->z=p0-2.5f*p1+2.0f*p2-0.5f*p3;
	v->w=-0.5f*p0+1.5f*p1-1.5f*p2+0.5f*p3;
}

void CAppForm::OnWire() 
{
	static DWORD s=D3DFILL_SOLID;
	s=(s==D3DFILL_WIREFRAME)?D3DFILL_SOLID:D3DFILL_WIREFRAME;
    m_pd3dDevice->SetRenderState( D3DRS_FILLMODE, s);

}

HRESULT CAppForm::CreateShader() 
{
	ID3DXBuffer *code=NULL,*err=NULL,*constants=NULL;
	HRESULT hr=D3DXAssembleShader(g_szShader,strlen(g_szShader),0,&constants,&code,&err);
	if (FAILED(hr)||err) {
		if (err) {
			if (err->GetBufferSize()) {
				char *errStr=(char *)err->GetBufferPointer();
				MessageBox(errStr);
			}
			err->Release();
		}
		return hr;
	}
	if (code) {
		DWORD dwDecl[] = {  D3DVSD_STREAM(0),
							D3DVSD_REG(0,  D3DVSDT_FLOAT4), 
							D3DVSD_REG(1,  D3DVSDT_FLOAT4), 
							D3DVSD_END() };
		int constSize=0;
		if (constants)
			constSize=constants->GetBufferSize();
		BYTE *dcl=new BYTE[sizeof(dwDecl)+constSize];
		memcpy(dcl+constSize,dwDecl,sizeof(dwDecl));
		if (constSize)
			memcpy(dcl,constants->GetBufferPointer(),constSize);
		if (constants)
			constants->Release();
		if (m_hVertexShader)
			m_pd3dDevice->DeleteVertexShader(m_hVertexShader);
		m_pd3dDevice->CreateVertexShader((DWORD *) dcl,(DWORD*) code->GetBufferPointer(),
			&m_hVertexShader,0);
		code->Release();
		delete dcl;
	}	
	return S_OK;
}

HRESULT CAppForm::RenderTiles(int numTiles,int numSamples)
{
	int imageWidth=m_d3dsdBackBuffer.Width*numTiles;
	int imageHeight=m_d3dsdBackBuffer.Height*numTiles;
	HRESULT hr;
    IDirect3DSurface8 *surf;
    if (FAILED(hr=m_pd3dDevice->CreateImageSurface(imageWidth*numSamples,
													imageHeight*numSamples,
//													D3DFMT_X8R8G8B8,
													m_d3dsdBackBuffer.Format,
													&surf))) 
		return hr;

	// save the old projection matrix
    D3DXMATRIX oldProj=m_matProj;

	numTiles*=numSamples;

	// scale the axii
    m_matProj._11*=numTiles;
    m_matProj._22*=numTiles;


	// loop through the tiles
	for (int i=0;i<numTiles;i++) {
		m_matProj._31=(numTiles-1)-i*2.0f;
		for (int j=0;j<numTiles;j++) {
			m_matProj._32=-((numTiles-1)-j*2.0f);
		    m_pd3dDevice->SetTransform(D3DTS_PROJECTION,&m_matProj);
			Render();
	        IDirect3DSurface8 *backbuf;
		    if(SUCCEEDED(hr=m_pd3dDevice->GetBackBuffer(0,D3DBACKBUFFER_TYPE_MONO,&backbuf))) {
					RECT destRect;
					destRect.left=m_d3dsdBackBuffer.Width*i;
					destRect.right=destRect.left+m_d3dsdBackBuffer.Width;
					destRect.top=m_d3dsdBackBuffer.Height*j;
					destRect.bottom=destRect.top+m_d3dsdBackBuffer.Height;
					if (FAILED(hr=D3DXLoadSurfaceFromSurface(surf,NULL,&destRect,backbuf,NULL,NULL,D3DX_FILTER_NONE,0))) {
						backbuf->Release();
						surf->Release();
						return hr;
					}
					backbuf->Release();
			} else {
				surf->Release();
				return hr;
			}
			m_pd3dDevice->Present(0,0,0,0);
		}
	}

	if (numSamples==1) {
	    hr=D3DXSaveSurfaceToFile("test3.bmp",D3DXIFF_BMP,surf,NULL,NULL);
		surf->Release();
		if (FAILED(hr)) return hr;
	} else {
		IDirect3DSurface8 *surf2;
		if (FAILED(hr=m_pd3dDevice->CreateImageSurface(imageWidth,
														imageHeight,
														D3DFMT_X8R8G8B8,
														&surf2))) {
			surf->Release();
			return hr;
		}

		if (FAILED(hr=D3DXLoadSurfaceFromSurface(surf2,NULL,NULL,surf,NULL,NULL,D3DX_DEFAULT,0))) {
			surf2->Release();
			surf->Release();
			return hr;
		}
		surf->Release();
		hr=D3DXSaveSurfaceToFile("test3.bmp",D3DXIFF_BMP,surf2,NULL,NULL);
		surf2->Release();
		if (FAILED(hr)) return hr;
	}

    // render normally after done saving
	m_matProj=oldProj;
    m_pd3dDevice->SetTransform(D3DTS_PROJECTION,&m_matProj);
    return Render();
}

HRESULT RenderTiled(LPCSTR fileName,LPDIRECT3DDEVICE8 pDev,int numTiles,HRESULT renderFunc())
{
	HRESULT hr;

	// get the backbuffer description
	LPDIRECT3DSURFACE8 backbuf;
    if(FAILED(hr=pDev->GetBackBuffer(0,D3DBACKBUFFER_TYPE_MONO,&backbuf))) 
		return hr;
	D3DSURFACE_DESC desc;
    hr=backbuf->GetDesc(&desc);
	backbuf->Release();
	if(FAILED(hr))
		return hr;

	// calculate final image size
	int imageWidth=desc.Width*numTiles;
	int imageHeight=desc.Height*numTiles;

	// create the image buffer
    LPDIRECT3DSURFACE8 surf;
    if (FAILED(hr=pDev->CreateImageSurface(imageWidth,
											imageHeight,
											desc.Format,
											&surf))) 
		return hr;

	// get the current projection matrix and save a copy
    D3DXMATRIX oldProj,newProj;
	pDev->GetTransform(D3DTS_PROJECTION,&oldProj);
	newProj=oldProj;

	// scale the projection matrix on x and y axii
    newProj._11*=numTiles;
    newProj._22*=numTiles;

	// loop through the tiles in X
	for (int i=0;i<numTiles;i++) {

		// offset x coordinates
		newProj._31=(numTiles-1)-i*2.0f;

		// loop throug the tiles in Y
		for (int j=0;j<numTiles;j++) {

			// offset y coordinates
			newProj._32=-((numTiles-1)-j*2.0f);

			// set the modified projection matrix
		    pDev->SetTransform(D3DTS_PROJECTION,&newProj);

			// call the rendering function
			if (FAILED(hr=renderFunc())) {
				surf->Release();
				return hr;
			}

			// get the back buffer pointer
		    if(FAILED(hr=pDev->GetBackBuffer(0,D3DBACKBUFFER_TYPE_MONO,&backbuf))) {
				surf->Release();
				return hr;
			}

			// copy the tile to the image buffer and release the back buffer
			RECT destRect;
			destRect.left=desc.Width*i;
			destRect.right=destRect.left+desc.Width;
			destRect.top=desc.Height*j;
			destRect.bottom=destRect.top+desc.Height;
			hr=D3DXLoadSurfaceFromSurface(surf,NULL,&destRect,backbuf,NULL,NULL,D3DX_FILTER_NONE,0);
			backbuf->Release();
			if (FAILED(hr)) {
				surf->Release();
				return hr;
			}

			// show current tile
			pDev->Present(0,0,0,0);
		}
	}

    // restore projection matrix
    pDev->SetTransform(D3DTS_PROJECTION,&oldProj);

	// save the image to specified file
    hr=D3DXSaveSurfaceToFile(fileName,D3DXIFF_BMP,surf,NULL,NULL);
	surf->Release();

	// return status of save to caller
	return hr;
}

//-----------------------------------------------------------------------------
// Name: CAppForm::Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------

BOOL g_captureScreen=FALSE;

void CAppForm::OnCapturescreen() 
{
	g_captureScreen=TRUE;	
}

CAppForm *cGappForm=NULL;

HRESULT rendFunc()
{
	if (cGappForm)
		return cGappForm->Render();
	return S_OK;
}

HRESULT CAppForm::Render()
{

//	m_pd3dDevice->GetTransform(D3DTS_PROJECTION,&m_matProj);
	if (g_captureScreen) {
		g_captureScreen=FALSE;
		return RenderTiles(10,1);
//		cGappForm=this;
//		return RenderTiled("test3.bmp",m_pd3dDevice,2,rendFunc);
	}

    // Clear the viewport
    m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
                         0x000000ff, 1.0f, 0L );

    // Begin the scene
    if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
    {
        // Render the vertex buffer contents
		m_pd3dDevice->SetIndices(m_pIB,0);
		if (m_hVertexShader) {
		    m_pd3dDevice->SetStreamSource( 0, m_pVB, sizeof(SPLINEVERTEX) );
			m_pd3dDevice->SetVertexShader(m_hVertexShader);
			D3DXMATRIX mat;
			D3DXMatrixMultiply( &mat, &m_matView, &m_matProj);

			D3DXMATRIX matScale,matTrans,mat2;
			D3DXMatrixScaling(&matScale,m_vQuadScaling.x,m_vQuadScaling.y,m_vQuadScaling.z);

//			D3DXMatrixMultiply(&mat,&matScale,&mat);

	//		D3DXMatrixTranspose( &mat, &mat );

			// shader input :	c0-c3	=	view/proj matrix

			m_pd3dDevice->SetVertexShaderConstant( 0, &mat, 4 );

			// shader input :	c4		=	patch scaling

//			m_pd3dDevice->SetVertexShaderConstant( 4, &m_vQuadScaling, 1 );

			// loop through the patches

			for (int i=0;i<m_numSplines-3;i++) {

				float baseZ=m_vQuadBase.z+i*m_fQuadSize;
				
				for (int j=0;j<m_numSplines-3;j++) {
	        
					// shader input :	c5		=	starting location

					float baseX=m_vQuadBase.x+j*m_fQuadSize;

		//			m_pd3dDevice->SetVertexShaderConstant( 5, &(D3DXVECTOR4(baseX,m_vQuadBase.y,baseZ,0.0f)), 1 );

					D3DXMatrixTranslation(&matTrans,baseX,m_vQuadBase.y,baseZ);
					D3DXMatrixMultiply(&mat2,&matScale,&matTrans);
					D3DXMatrixMultiply(&mat2,&mat2,&mat);
					D3DXMatrixTranspose( &mat2, &mat2 );
					// shader input :	c0-c3	=	view/proj matrix
					m_pd3dDevice->SetVertexShaderConstant( 0, &mat2, 4 );

					// shader input :	c6-c9	=	patch altitude matrix

					m_pd3dDevice->SetVertexShaderConstant( 6, &splineMat[i][j], 4 );

					// shader input :	c16-c19	=	patch lighting matrix

					m_pd3dDevice->SetVertexShaderConstant( 10, &splineNormMat[i][j], 4 );

					// render the patch

					m_pd3dDevice->DrawIndexedPrimitive( D3DPT_TRIANGLELIST, 0, m_patchWidth*m_patchWidth, 0, (m_patchWidth-1)*(m_patchWidth-1)*2);			
				}
			}
		}

        // Render stats and help text  
        RenderText();

        // End the scene.
        m_pd3dDevice->EndScene();
    }

    return S_OK;
}

float CAppForm::CalcQuad(float x,float z,D3DXMATRIX *quad)
{
	D3DXVECTOR4 v1,v2;
	v1=D3DXVECTOR4(1.0f,x,x*x,x*x*x);
	v2=D3DXVECTOR4(1.0f,z,z*z,z*z*z);
	D3DXVec4Transform(&v2,&v2,quad);
	return D3DXVec4Dot(&v1,&v2);
}

#define NUM_OCTAVES		4

void CAppForm::CreateGeometry(BOOL bNewLand)
{
	int i,j,k;
	float maxVal,minVal;
	float scale;
	
	if (!m_terrainFile)
		m_terrainFile=new CTerrainFile("land.raw",512,512);

	static int startX[NUM_OCTAVES];
	static int startY[NUM_OCTAVES];
	if (bNewLand) {
		srand(time(NULL)+rand());
		for (i=0;i<NUM_OCTAVES;i++) {
			startX[i]=rand()%(512-MAX_SPLINES-4)+1;
			startY[i]=rand()%(512-MAX_SPLINES-3);
		}
		InitViewOrientation();
	}

	m_terrainFile->GetHeightsToArray(m_fHeight[0],startX[0],startY[1],MAX_SPLINES,MAX_SPLINES,m_numSplines,m_numSplines,MAX_SPLINES);
	float tempHeight[MAX_SPLINES][MAX_SPLINES];
	int *index1=&k;
	int *index2=&j;
	float octScale=0.5f;
	for (i=1;i<NUM_OCTAVES;i++) {
		m_terrainFile->GetHeightsToArray(tempHeight[0],startX[i],startY[i],MAX_SPLINES,MAX_SPLINES,m_numSplines,m_numSplines,MAX_SPLINES);
		for (j=0;j<m_numSplines;j++)
			for (k=0;k<m_numSplines;k++)
				m_fHeight[*index1][*index2]+=tempHeight[j][k]*octScale*m_fHeight[*index1][*index2];
		octScale*=0.5f;
		int *tempPtr=index1;
		index1=index2;
		index2=tempPtr;
	}

	maxVal=minVal=m_fHeight[1][1];
	for (i=1;i<m_numSplines-1;i++) {
		for (j=1;j<m_numSplines-1;j++) {
			if (m_fHeight[i][j]>maxVal)
				maxVal=m_fHeight[i][j];
			if (m_fHeight[i][j]<minVal)
				minVal=m_fHeight[i][j];
		}
		m_yOffset[i]=minVal;
		m_yScaling[i]=maxVal;
	}
	scale=1.0f/(maxVal-minVal);
	for (i=0;i<m_numSplines;i++) {
		for (j=0;j<m_numSplines;j++) {
			m_fHeight[i][j]-=minVal;
			m_fHeight[i][j]*=scale;
			m_fHeight[i][j]=(1.0f-m_fHeight[i][j])*(1.0f-m_fHeight[i][j]);
		}
		float areaMin=(m_yOffset[i]-minVal)*scale;
		float areaMax=(m_yScaling[i]-minVal)*scale;
		m_yScaling[i]=LAND_Y_SCALE/(areaMax-areaMin);
		m_yOffset[i]=(-areaMin/(areaMax-areaMin))*LAND_Y_SCALE;
	}

	// pre-calculate matrices for vertex position within patches

	CalcQuadMatrices(splineMat[0],m_fHeight[0],m_numSplines,m_numSplines);
	
	CalcQuadScaling();

	D3DXVECTOR3 vLight=D3DXVECTOR3(1.0f,-1.0f,1.0f);
	D3DXVec3Normalize(&vLight,&vLight);
	for (i=0;i<m_numSplines;i++) {
		for (j=0;j<m_numSplines;j++) {
			D3DXVECTOR3 vx,vz;
			D3DXVECTOR3 vSum=D3DXVECTOR3(0.0f,0.0f,0.0f);
			D3DXVECTOR3 norm;
			int normCnt=0;
			if (i>1) {
				vx=D3DXVECTOR3(-m_fQuadSize,
							   m_vQuadScaling.y*(m_fHeight[i-1][j]-m_fHeight[i][j]),
							   0.0f);
				if (j>1) {
					vz=D3DXVECTOR3(0.0f,
								   m_vQuadScaling.y*(m_fHeight[i][j-1]-m_fHeight[i][j]),
								   -m_fQuadSize);
					D3DXVec3Normalize(&vx,&vx);
					D3DXVec3Normalize(&vz,&vz);
					D3DXVec3Cross(&norm,&vx,&vz);
					D3DXVec3Normalize(&norm,&norm);
					vSum+=norm;
					normCnt++;
				}
				if (j<m_numSplines-1) {
					vz=D3DXVECTOR3(0.0f,
								   m_vQuadScaling.y*(m_fHeight[i][j+1]-m_fHeight[i][j]),
								   m_fQuadSize);
					D3DXVec3Normalize(&vx,&vx);
					D3DXVec3Normalize(&vz,&vz);
					D3DXVec3Cross(&norm,&vz,&vx);
					D3DXVec3Normalize(&norm,&norm);
					vSum+=norm;
					normCnt++;
				}
			}
			if (i<m_numSplines-1) {
				vx=D3DXVECTOR3(m_fQuadSize,
							   m_vQuadScaling.y*(m_fHeight[i+1][j]-m_fHeight[i][j]),
							   0.0f);
				if (j>1) {
					vz=D3DXVECTOR3(0.0f,
								   m_vQuadScaling.y*(m_fHeight[i][j-1]-m_fHeight[i][j]),
								   -m_fQuadSize);
					D3DXVec3Normalize(&vx,&vx);
					D3DXVec3Normalize(&vz,&vz);
					D3DXVec3Cross(&norm,&vz,&vx);
					D3DXVec3Normalize(&norm,&norm);
					vSum+=norm;
					normCnt++;
				}
				if (j<m_numSplines-1) {
					vz=D3DXVECTOR3(0.0f,
								   m_vQuadScaling.y*(m_fHeight[i][j+1]-m_fHeight[i][j]),
								   m_fQuadSize);
					D3DXVec3Normalize(&vx,&vx);
					D3DXVec3Normalize(&vz,&vz);
					D3DXVec3Cross(&norm,&vx,&vz);
					D3DXVec3Normalize(&norm,&norm);
					vSum+=norm;
					normCnt++;
				}
			}
			norm=vSum/(float)normCnt;
			D3DXVec3Normalize(&norm,&norm);
			m_fLight[i][j]=0.2f+0.8f*max(D3DXVec3Dot(&norm,&vLight),0.0f);
		}
	}

	// pre-calculate matrices for lighting of patches

	CalcQuadMatrices(splineNormMat[0],m_fLight[0],m_numSplines,m_numSplines);
}

void CAppForm::OnNewLandscape() 
{
	CreateGeometry();	
}

void CAppForm::OnLargerpatches() 
{
	if (m_patchWidth<64) {
		m_patchWidth++;
		m_fTextUpdateTime=0.0f;
		CreateBuffers();
	}
}

void CAppForm::OnSmallerpatches() 
{
	if (m_patchWidth>2) {
		m_patchWidth--;
		m_fTextUpdateTime=0.0f;
		CreateBuffers();
	}
}

void CAppForm::OnMoresplines() 
{
	if (m_numSplines<MAX_SPLINES) {
		m_numSplines++;
		m_fTextUpdateTime=0.0f;
		CreateGeometry(FALSE);
	}
}

void CAppForm::OnLesssplines() 
{
	if (m_numSplines>4) {
		m_numSplines--;
		m_fTextUpdateTime=0.0f;
		CreateGeometry(FALSE);
	}
}

//-----------------------------------------------------------------------------
// Name: CAppForm::FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------

HRESULT CAppForm::FrameMove()
{
	// Update the world state 
	D3DXMATRIX matRotY;
	D3DXMATRIX matRotX,matRot,matTrans;

	if (m_bAutoRot)
	   m_fRotY += m_fElapsedTime;

	D3DXMATRIX mat;
	D3DXMatrixRotationY(&mat,m_fRotY);
	D3DXVECTOR3 vCam;
	D3DXVec3TransformCoord(&vCam,&D3DXVECTOR3(0.0f,LAND_Y_SCALE*2.6f,LAND_XZ_SCALE*1.3f),&mat);
	D3DXMatrixLookAtLH(&m_matView,
					   &vCam,
					   &D3DXVECTOR3(0.0f,LAND_Y_SCALE*0.25f,0.0f),
					   &D3DXVECTOR3(0.0f,1.0f,0.0f));

    return S_OK;
}

// function to pre-calculate matrices for a block of control points

void CAppForm::CalcQuadMatrices(D3DXMATRIX *pMat,float *pData,int numRows,int numCols)
{
	// Catmull-Rom Basis Matrix

	D3DXMATRIX mat;
	mat._11=0.0;	mat._12=1.0;	mat._13=0.0;	mat._14=0.0;
	mat._21=-0.5;	mat._22=0.0;	mat._23=0.5;	mat._24=0.0;
	mat._31=1.0;	mat._32=-2.5;	mat._33=2.0;	mat._34=-0.5;
	mat._41=-0.5;	mat._42=1.5;	mat._43=-1.5;	mat._44=0.5;

	// loop through the rows

	for (int i=0;i<numRows-3;i++) {

		// loop through the columns

		for (int j=0;j<numCols-3;j++) {

			// get a pointer to the target matrix

			float *ptr=pMat[i*(MAX_SPLINES-3)+j];

			// loop through the four spline segments defining this patch

			for (int k=0;k<4;k++) {

				// calculate 4D vector for this segment

				PrecalcCatmullRom((D3DXVECTOR4 *) ptr,
						   pData[(i+k)*MAX_SPLINES+j],
						   pData[(i+k)*MAX_SPLINES+j+1],
						   pData[(i+k)*MAX_SPLINES+j+2],
						   pData[(i+k)*MAX_SPLINES+j+3]);
				ptr+=4;
			}

			// post-multiply by basis matrix

			D3DXMatrixMultiply(&pMat[i*(MAX_SPLINES-3)+j],&mat,&pMat[i*(MAX_SPLINES-3)+j]);
		}
	}
}

void CAppForm::CalcQuadScaling()
{
	m_fQuadSize=LAND_XZ_SCALE/(float)(m_numSplines-3);
	m_vQuadScaling=D3DXVECTOR4(m_fQuadSize,m_yScaling[m_numSplines-2],m_fQuadSize,1.0f);
	m_vQuadBase=D3DXVECTOR4(-(m_numSplines-3)*m_fQuadSize*0.5f,m_yOffset[m_numSplines-2],-(m_numSplines-3)*m_fQuadSize*0.5f,0.0f);
}

float CAppForm::CalcHeight(float x, float z)
{
	// offset to base

	x-=m_vQuadBase.x;
	z-=m_vQuadBase.z;

	// normalize to quad size

	x/=m_fQuadSize;
	z/=m_fQuadSize;

	// get integer part

	int row=(int) z;
	int col=(int) x;

	// return if out of range

	if (row<0||row>=m_numSplines-3||col<0||col>=m_numSplines-3) {
		return 0.0f;
	}

	// get fractional parts

	x-=floorf(x);
	z-=floorf(z);

	// calculate and return quad height

	return m_vQuadScaling.y*CalcQuad(x,z,&splineMat[row][col])+m_vQuadBase.y;
}

void CAppForm::InitViewOrientation()
{
	m_fRotY                = D3DX_PI*0.25f;
	m_vPos=D3DXVECTOR3(0.0f,0.0f,0.0f);
}
